/** LibrasEditor_.java:
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
 @author Hemerson Pistori (pistori@ec.ucdb.br)
 @created 18 de Maro de 2004
 
*/

package sigus.apps.libras;

import sigus.machineLearning.*;
import sigus.core.*;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.net.*;

import ij.*;
import ij.plugin.filter.PlugInFilter;
import ij.process.*;
import ij.gui.*;

import javax.swing.*;

/**
 
   A simple text area for one digit at a time presentation. It is just a
   demo for LIBRAS hand sign recognition.
   
*/
public class LibrasEditor_ extends JFrame implements SigusApp, PlugInFilter, MouseListener, KeyListener {

    Updater updater = null;
    ImageProcessor ip = null;
    boolean calibrating = true;

    JTextArea textArea;

    /** Initialize graphical inteface componentes */
    public void init () {
        this.setDefaultCloseOperation (JFrame.EXIT_ON_CLOSE);

        //add Mouse and Key Listener (this);
        addMouseListener (this);


        // Create Text Aread
        textArea = new JTextArea(1,1);
        textArea.setFont(new Font("Times", Font.PLAIN, 350));
        textArea.setLineWrap(true);
        textArea.setWrapStyleWord(true);

        textArea.addKeyListener(this);

        textArea.setAlignmentX(Component.LEFT_ALIGNMENT);

        JLabel jLabelMinus = new JLabel("Type minus (-) to start/stop calibration mode");
        jLabelMinus.setAlignmentX(Component.LEFT_ALIGNMENT);

        JLabel jLabelDot = new JLabel("Type dot (.) to stop training and start testing");
        jLabelDot.setAlignmentX(Component.LEFT_ALIGNMENT);

        Container contentPane = getContentPane();
        contentPane.setLayout(new BoxLayout(contentPane, BoxLayout.PAGE_AXIS));
        contentPane.add(textArea);
        contentPane.add(jLabelMinus);
        contentPane.add(jLabelDot);


        setTitleTraining();


        // Set Bounds
        //        Toolkit tk = Toolkit.getDefaultToolkit ();
        //        Dimension d = tk.getScreenSize ();
        //        setBounds (400, 0,d.width-500, d.height-150);

        setLocation(450,50);

        pack();
        setVisible ( true );
        //show();
    }

    public void setGeneralTitle(String title) {
        setTitle ( title );
    }

    public void setTitleTraining() {
        setTitle ( "Libras Editor (TRAINING - Look and Click)" );
    }

    public void setTitleLearning() {
        setTitle ( "Libras Editor (TRAINING - Click Again to stop collecting samples)" );
    }

    public void setTitlePlaying() {
        if( ! updater.isLearning() )
            setTitleTraining();
        else
            setTitle ( "Libras Editor (PLAYING - Just Look)" );
    }


    public void destroy () {
        removeMouseListener (this);
        textArea.removeKeyListener (this);
    }


    /**
    Handle keyboard typing. If the key is a minus sign ('-'), the application interchanges between calibration and non calibration mode. 
    */
    public void keyTyped(KeyEvent e) {
        char keyin = e.getKeyChar();
        // add some code here

        if(keyin == '-') {
            calibrating = !calibrating;
            return;
        }


        if(updater != null) {
            if( ! updater.isUpdating() ) {
                if(keyin == '.')
                    updater.setStopRequested(true);
                else {
                    setTitleLearning();
                    updater.startUpdating(""+keyin);
                }
            } else {
                setTitlePlaying();
                updater.stopUpdating();
            }
        }

        textArea.setText("");

    }

    public void process(int n) {

        char [] temp = new char[1];

        if(n > 0 ) {

            temp[0] = (char) ('a'+(n-1));
            textArea.setText((new String(temp)));
        } else {
            textArea.setText(" ");
        }
        //	    textArea.setCaretPosition(textArea.getDocument().getLength());


    }


    public void keyReleased(KeyEvent e) {
        //add some code here
    }
    public void keyPressed(KeyEvent e) {
        // add some code here
    }



    public void mouseReleased (MouseEvent event) {
        /*			int x = event.getX ();
        			int y = event.getY ();
         
        			Dimension d = getSize ();
        			int c = (x * 3) / d.width;
        			int r = (y * 3) / d.height;
        //			process(c,r);
        //			System.out.println("antes update");
                    if(updater != null) {
        			   if( ! updater.isUpdating() ) {
        				  setTitleLearning();
        			      updater.startUpdating(getSequencial(c,r));
        		       } else
        		       {
        				  setTitlePlaying();
        			      updater.stopUpdating();
        		       }
        		    }*/
    }


    public void trainingFinished() {
        setTitlePlaying();
    }

    public boolean getCalibrating() {
        return calibrating;
    }

    public void mousePressed (MouseEvent e) {}

    public void mouseEntered (MouseEvent e) {}

    public void mouseExited (MouseEvent e) {}

    public void mouseClicked ( MouseEvent e) {}



    // public String getAppletInfo() {
    //return "LibrasEditor by Arthur van Hoff";
    //}
    ImagePlus imp;
    public static void main (String args[]) {
        LibrasEditor_ lib = new LibrasEditor_ ();
        lib.init ();
    }

    /** Filters use this method to process the image. If the
     *    SUPPORTS_STACKS flag was set, it is called for each slice in
     *    a stack. ImageJ will lock the image before calling
     *   this method and unlock it when the filter is finished.
     */
    public void run (ImageProcessor ip) {
        LibrasEditor_ lib = this;
        lib.init();
    }


    public void setUpdater( Updater u) {
        updater = u;
    }


    public void setImageProcessor( ImageProcessor i) {
        ip = i;
    }


    /** This method is called once when the filter is loaded. 'arg',
     *   which may be blank, is the argument specified for this plugin
     *   in IJ_Props.txt. 'imp' is the currently active image.
     *   This method should return a flag word that specifies the
     *   filters capabilities.
     */
    public int setup (String arg, ImagePlus imp) {
        this.imp = imp;
        if (arg.equals("about")) {
            showAbout();
            return DONE;
        }
        return NO_IMAGE_REQUIRED;
    }

    void showAbout() {
        IJ.showMessage("About Libras Editor...",
                       "Libras Editor\n");
    }

}
