/** TTTEffect_.java:
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
 @author Hemerson Pistori (pistori@ec.ucdb.br)
 @created 18 de Maro de 2004
 
*/

package sigus.apps.ttt;

import sigus.core.*;
import sigus.segmentation.*;
import sigus.parameterExtraction.*;
import sigus.tracking.*;


import ij.*;

/**
 *  Effect used by the TTT application.
 *
 * @author    Hemerson Pistori (pistori@ec.ucdb.br)
 * @created    18 de Maro de 2004
 *
 */
public class TTTEffect extends SigusEffect {

    public TTTEffect() {}

    ColorBasedSegmentation segmenter = new ColorBasedSegmentation();
    ImageMomentsExtractor_ extractor = new ImageMomentsExtractor_();
    ImageMomentsTracking_ tracker = new ImageMomentsTracking_();

    public void init() {
        MAX_CLASS = 10; // Number of different training class values + 1
        maxFocus = 3;
        minInstances = 5000;
        saveInstancesImages = false;
    }


    public void applyFilters(byte[] inData, byte[] jmfData, byte[] outData, int width, int height) {

        if(isCalibrating()) {
            segmenter.extractSamples(inData, jmfData, outData, width, height);
        } else {
            segmenter.segment(inData, jmfData, outData, width, height);
            IJ.run("Threshold");
            tracker.setup("",imp);
            tracker.run(ip);
        }

    }


    public void createParameterExtractor() {
        pe = new ImageMomentsExtractor_();
    }


    public void createUpdater() {
        updater = new UpdaterTTT_();
    }


    public void createSigusApp() {
        sigusApp = new TicTacToe_(this);
    }

}
