/**
 *  TicTacToe_.java:
 *
 * @version 	1.2, 13 Oct 1995
 * @author Arthur van Hoff
 * @modified 04/23/96 Jim Hagen : winning sounds
 * @modified 02/10/98 Mike McCloskey : added destroy()
 * @modified 27/07/02 Hemerson Pistori 
 *                    : Turned into an ImageJ Plugin
 */

package sigus.apps.ttt;

import sigus.machineLearning.*;
import sigus.core.*;

import java.awt.event.MouseEvent;
import javax.swing.event.MouseInputListener;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.net.*;

import ij.*;
import ij.plugin.filter.PlugInFilter;
import ij.process.*;
import ij.gui.*;



import javax.swing.*;

public class TicTacToe_ extends JFrame implements SigusApp, PlugInFilter {

    /** An updater collects training samples */
    Updater updater = null;
    /** A SigusEffect process images captured by the Webcam and controll training and testing phases */
    SigusEffect effect = null;
    /** ImageProcessor and ImagePlus take control tof he window where the processed image is shown */
    ImageProcessor ip = null;
    /** ImageProcessor and ImagePlus take control tof he window where the processed image is shown */
    ImagePlus imp;
    /** True if tye system is collecting skin (or pixels of the target object - hands, eyes, face, etc) samples */
    boolean calibrating = true;


    /**
     * White's (Not - Circle) current position. The computer is white.
     */
    int white;


    /**
     * Black's (Cross) current position. The user is black.
     */
    int black;

    /**
     * The squares in order of importance...
     */
    final static int moves[] =
        {4, 0, 2, 6, 8, 1, 3, 5, 7};

    /**
     * The winning positions.
     */
    static boolean won[] = new boolean[1 << 9];
    static final  int DONE = (1 << 9) - 1;
    static final  int OK = 0;
    static final  int WIN = 1;
    static final  int LOSE = 2;
    static final  int STALEMATE = 3;



    public TicTacToe_(SigusEffect effect) {
        this.effect = effect;
    }


    /**
     * Mark all positions with these bits set as winning.
     */
    private static void isWon (int pos) {
        for (int i = 0 ; i < DONE ; i++) {
            if ((i & pos) == pos) {
                won[i] = true;
            }
        }
    }

    /** Restart the game */
    private void resetGame() {
        white = 0;
        black = 0;
        repaint();
    }

    /**
     * Initialize all winning positions.
     */
    static
    {
        isWon ((1 << 0) | (1 << 1) | (1 << 2));
        isWon ((1 << 3) | (1 << 4) | (1 << 5));
        isWon ((1 << 6) | (1 << 7) | (1 << 8));
        isWon ((1 << 0) | (1 << 3) | (1 << 6));
        isWon ((1 << 1) | (1 << 4) | (1 << 7));
        isWon ((1 << 2) | (1 << 5) | (1 << 8));
        isWon ((1 << 0) | (1 << 4) | (1 << 8));
        isWon ((1 << 2) | (1 << 4) | (1 << 6));
    }

    /**
     * Compute the best move for white (computer)
     * @return the square to take
     */
    int bestMove (int white, int black) {
        int bestmove = -1;

loop:
        for (int i = 0 ; i < 9 ; i++) {
            int mw = moves[i];
            if (((white & (1 << mw)) == 0) && ((black & (1 << mw)) == 0)) {
                int pw = white | (1 << mw);
                if (won[pw]) {
                    // white wins, take it!
                    return mw;
                }
                for (int mb = 0 ; mb < 9 ; mb++) {
                    if (((pw & (1 << mb)) == 0) && ((black & (1 << mb)) == 0)) {
                        int pb = black | (1 << mb);
                        if (won[pb]) {
                            // black wins, take another
                            continue loop;
                        }
                    }
                }
                // Neither white nor black can win in one move, this will do.
                if (bestmove == -1) {
                    bestmove = mw;
                }
            }
        }
        if (bestmove != -1) {
            return bestmove;
        }

        // No move is totally satisfactory, try the first one that is open
        for (int i = 0 ; i < 9 ; i++) {
            int mw = moves[i];
            if (((white & (1 << mw)) == 0) && ((black & (1 << mw)) == 0)) {
                return mw;
            }
        }

        // No more moves
        return -1;
    }

    /**
     * User move.
     * @return true if legal
     */
    boolean yourMove (int m) {
        if ((m < 0) || (m > 8)) {
            return false;
        }
        if (((black | white) & (1 << m)) != 0) {
            return false;
        }
        black |= 1 << m;
        return true;
    }

    /**
     * Computer move.
     * @return true if legal
     */
    boolean myMove () {
        if ((black | white) == DONE) {
            return false;
        }
        int best = bestMove (white, black);
        white |= 1 << best;
        return true;
    }

    /**
     * Figure what the status of the game is.
     */
    int status () {
        if (won[white]) {
            return WIN;
        }
        if (won[black]) {
            return LOSE;
        }
        if ((black | white) == DONE) {
            return STALEMATE;
        }
        return OK;
    }

    /**
     * Who goes first in the next game?
     */
    boolean first = true;


    /**
     * Initialize the applet. Resize and load images.
     */
    public void init () {
        this.setDefaultCloseOperation (JFrame.EXIT_ON_CLOSE);


        Container container = this.getContentPane();
        container.setLayout(new BoxLayout(container,
                                          BoxLayout.PAGE_AXIS));

        GraphicArea graphicArea = new GraphicArea(this);
        container.add(graphicArea);

        JButton jCalibrateButton = new javax.swing.JButton();
        jCalibrateButton.setText("Start/Stop Calibrating");
        jCalibrateButton.setToolTipText("Start/Stop Calibration");
        jCalibrateButton.addActionListener(new java.awt.event.ActionListener() {
                                               public void actionPerformed(java.awt.event.ActionEvent evt) {
                                                   jCalibrateButtonClicked(evt);
                                               }
                                           }
                                          );


        JButton jTrainingButton = new javax.swing.JButton();
        jTrainingButton.setText("Stop Training");
        jTrainingButton.setToolTipText("Stop Training");
        jTrainingButton.addActionListener(new java.awt.event.ActionListener() {
                                              public void actionPerformed(java.awt.event.ActionEvent evt) {
                                                  jTrainingButtonClicked(evt);
                                              }
                                          }
                                         );

        JButton jClearExamplesButton = new javax.swing.JButton();
        jClearExamplesButton.setText("Clear Previous Examples");
        jClearExamplesButton.setToolTipText("Clear Examples and Restart Training");
        jClearExamplesButton.addActionListener(new java.awt.event.ActionListener() {
                                                   public void actionPerformed(java.awt.event.ActionEvent evt) {
                                                       jClearExamplesButtonClicked(evt);
                                                   }
                                               }
                                              );

        JPanel buttons = new JPanel();

        buttons.add(jCalibrateButton);
        buttons.add(jTrainingButton);
        buttons.add(jClearExamplesButton);

        graphicArea.setAlignmentX(Component.LEFT_ALIGNMENT);
        buttons.setAlignmentX(Component.LEFT_ALIGNMENT);

        container.add(buttons);

        setTitleTraining();
        setLocation(300,100);
        pack();
        setVisible ( true );

    }

    /** Switch the calibration phase on and off */
    private void jCalibrateButtonClicked(java.awt.event.ActionEvent evt) {
        calibrating = !calibrating;
    }

    /** Stop training phase and start playing by vision  */
    private void jTrainingButtonClicked(java.awt.event.ActionEvent evt) {
        if(updater != null) {
            if(!updater.isUpdating())
                updater.setStopRequested(true);
        }

    }

    /** Clear the previously collected examples and restart the training phase */
    private void jClearExamplesButtonClicked(java.awt.event.ActionEvent evt) {
        if(updater != null)
            updater.clearTrainingSet();
        effect.resetLearner();
        resetGame();
    }

    public void setTitleTraining() {
        setTitle ( "Tic Tac Toe (TRAINING - Look and Click) [Type dot (.) to start playing]" );
    }

    public void setTitleLearning() {
        setTitle ( "Tic Tac Toe (TRAINING - Click Again to stop collecting samples)" );
    }

    public void setGeneralTitle(String title) {
        setTitle (title);
    }

    public void setTitlePlaying() {
        if( ! updater.isLearning() )
            setTitleTraining();
        else
            setTitle ( "Tic Tac Toe (PLAYING - Just Look)" );
    }


    /** Get a number indicating the square that the player looked at and execute the proper play */
    public void process(int n) {

        if(n == 1)
            process(0,0);
        else if(n == 2)
            process(1,0);
        else if(n == 3)
            process(2,0);
        else if(n == 4)
            process(0,1);
        else if(n == 5)
            process(1,1);
        else if(n == 6)
            process(2,1);
        else if(n == 7)
            process(0,2);
        else if(n == 8)
            process(1,2);
        else if(n == 9)
            process(2,2);
    }

    /** Get the column and row and returns a sequencial number indicating the square that the player looked at */
    private String getSequencial(int c, int r) {
        if(c == 0) {
            if(r == 0)
                return("1");
            else if(r == 1)
                return("4");
            else if(r == 2)
                return("7");
        } else if(c == 1) {
            if(r == 0)
                return("2");
            else if(r == 1)
                return("5");
            else if(r == 2)
                return("8");
        } else if(c == 2) {
            if(r == 0)
                return("3");
            else if(r == 1)
                return("6");
            else if(r == 2)
                return("9");
        }
        return("0");
    }

    /** Process the user play */
    public void process(int c, int r) {

        switch (status ()) {
        case WIN:
        case LOSE:
        case STALEMATE:
            //play (getCodeBase (), "audio/return.au");
            white = black = 0;
            /*					if (first)
            					{
            						white |= 1 << (int)(Math.random () * 9);
            					}
            					first = !first;*/
            repaint ();
            return;
        }

        // Figure out the row/column
        //			System.out.println("c = "+c+" r = "+r);
        if (yourMove (c + r * 3)) {
            repaint ();

            switch (status ()) {
            case WIN:
                //play (getCodeBase (), "audio/yahoo1.au");
                break;
            case LOSE:
                //play (getCodeBase (), "audio/yahoo2.au");
                break;
            case STALEMATE:
                break;
            default:
                if (myMove ()) {
                    //		try { Thread.sleep(3000); } catch (InterruptedException interruptedException){}

                    repaint ();
                    switch (status ()) {
                    case WIN:
                        //play (getCodeBase (), "audio/yahoo1.au");
                        break;
                    case LOSE:
                        //play (getCodeBase (), "audio/yahoo2.au");
                        break;
                    case STALEMATE:
                        break;
                    default:
                        //play (getCodeBase (), "audio/ding.au");
                    }
                } else {
                    //play (getCodeBase (), "audio/beep.au");
                }
            }
        } else {
            //play (getCodeBase (), "audio/beep.au");
        }
    }

    public void trainingFinished() {
        setTitlePlaying();
    }


    public boolean getCalibrating() {
        return calibrating;
    }


    /** Calls updater to collected a training sample (the user looked and clicked)*/
    public void mouseClicked(int c, int r) {

        if(updater != null) {
            if( ! updater.isUpdating() ) {
                setTitleLearning();
                updater.startUpdating(getSequencial(c,r));
            } else {
                setTitlePlaying();
                updater.stopUpdating();
            }
        }
    }




    public void run (ImageProcessor ip) {
        TicTacToe_ tic = this;
        tic.init();
    }


    public void setUpdater( Updater u) {
        updater = u;
    }


    public void setImageProcessor( ImageProcessor i) {
        ip = i;
    }




    /** This method is called once when the filter is loaded. 'arg',
     *   which may be blank, is the argument specified for this plugin
     *   in IJ_Props.txt. 'imp' is the currently active image.
     *   This method should return a flag word that specifies the
     *   filters capabilities.
     */
    public int setup (String arg, ImagePlus imp) {
        this.imp = imp;
        if (arg.equals("about")) {
            showAbout();
            return DONE;
        }
        return NO_IMAGE_REQUIRED;
    }

    void showAbout() {
        IJ.showMessage("About Tic Tac Toe...",
                       "Tic Tac Toe Game\n");
    }



    public static class GraphicArea extends JComponent
        implements MouseInputListener {

        /** Relative path of the directory where the cross and not images are  located.*/
        String imagesDir = "sigus/apps/ttt/images/";
        Image notImage;
        Image crossImage;
        TicTacToe_ controller;

        Dimension preferredSize = new Dimension(600,600);

        public GraphicArea(TicTacToe_ controller) {

            this.controller = controller;
            Toolkit tk = Toolkit.getDefaultToolkit ();
            notImage = tk.getImage (imagesDir+"not.gif");
            crossImage = tk.getImage (imagesDir+"cross.gif");

            MediaTracker mediatracker = new MediaTracker(this);
            mediatracker.addImage(notImage,0);
            mediatracker.addImage(crossImage,0);
            try {
                mediatracker.waitForAll();
            } catch (InterruptedException interruptedException) {
                interruptedException.printStackTrace ();
            }


            addMouseListener(this);
        }

        public Dimension getPreferredSize() {
            return preferredSize;
        }


        /**
         * Paint the current configuration of a TTT Game (positions of nots and crosses)
        */
        public void paintComponent(Graphics g) {

            Dimension d = getSize ();
            g.setColor (Color.black);
            int xoff = d.width / 3;
            int yoff = d.height / 3;

            int meiox = d.width/2-xoff;
            int meioy = d.height/2-yoff;
            g.drawLine (xoff, 0, xoff, d.height);
            g.drawLine (2*xoff, 0, 2*xoff, d.height);
            g.drawLine (0, yoff, d.width, yoff);
            g.drawLine (0, 2*yoff, d.width, 2*yoff);

            int i = 0;
            for (int r = 0 ; r < 3 ; r++) {
                for (int c = 0 ; c < 3 ; c++, i++) {
                    if ((controller.white & (1 << i)) != 0) {
                        g.drawImage (notImage, meiox+c*xoff , meioy+r*yoff , this);
                    } else if ((controller.black & (1 << i)) != 0) {
                        g.drawImage (crossImage, meiox+c*xoff , meioy+ r*yoff , this);
                    }
                }
            }
        }


        public void mouseClicked(MouseEvent event) {
            int x = event.getX ();
            int y = event.getY ();

            Dimension d = getSize ();
            int c = (x * 3) / d.width;
            int r = (y * 3) / d.height;

            controller.mouseClicked(c,r);
            repaint();
        }

        public void mouseMoved(MouseEvent e) {}
        public void mouseExited(MouseEvent e) {}
        public void mouseReleased(MouseEvent e) { }
        public void mouseEntered(MouseEvent e) { }
        public void mousePressed(MouseEvent e) { }
        public void mouseDragged(MouseEvent e) { }
    }


}
