package sigus.camera;

import sigus.*;


import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.media.*;
import javax.media.control.TrackControl;
import javax.media.Format;
import javax.media.format.*;
import javax.swing.*;


import ij.*;
import ij.plugin.filter.PlugInFilter;
import ij.process.*;
import ij.gui.*;

/**
 *  Reads a flow of frames from a webcam using the JMF package (javax.media.*).
 *  
 */
public class CameraReader extends JFrame implements ControllerListener,PlugInFilter {

    Processor  p;
    Object     waitSync           = new Object ();
    boolean    stateTransitionOK  = true;
    //    ImagePlus imp = null;
    Codec      codec = null;
    private static String	defaultEncoding = "YUV";
	private static String   defaultWidth = "320";
	private static String   defaultHeight = "240";
	private static String   defaultDataLength = "115200";
	private static String   defaultFrameRate = "15";
	

    public int setup(String arg, ImagePlus imp) {
        //        this.imp = imp;
        if (arg.equals("about")) {
            showAbout();
            return DONE;
        }
        return NO_IMAGE_REQUIRED;
    }

    public void run(ImageProcessor ip) {}



    void showAbout() {
        IJ.showMessage("About Tic Tac Toe...",
                       "This plugin finds n circles of fixed radius\n" +
                       "using a basic HoughTransform operator\n." +
                       "For better results apply an Edge Detector\n" +
                       "filter and a binarizer before using this plugin\n"+
                       "\nAuthor: Hemerson Pistori (pistori@ec.ucdb.br)"
                      );
    }


    public CameraReader( Codec pcodec) {
        //		super ( "JMF" );

        setCodec(pcodec);
        Init();
    }



    public void Init() {
        MediaLocator       ml;
        CaptureDeviceInfo  cd;
        Vector             li;
        CaptureDeviceInfo   infoCaptureDevice;
        Format              arrFormats [];

        li = CaptureDeviceManager.getDeviceList ( new VideoFormat ( "YUV" ) );


        if( li.size() == 0 ) {
            li = CaptureDeviceManager.getDeviceList ( new VideoFormat ( "RGB" ) );
        }

        if( li.size() == 0 ) {
            System.out.println("Nao esta encontrando driver da camera");
            System.exit(-1);
        }
		 
 
       		
		// Mostra a Lista de Dispositivos Encontrados
		//for(int i = 0; i < li.size(); ++i) {
		//   System.out.println("Dispositivo "+i);
    	//   System.out.println(li.elementAt(i).toString());
		//}

        cd = (CaptureDeviceInfo) li.remove ( 0 );
        if ( cd == null ) {
            System.err.println ( "Cannot build media locator for camera" );
            System.exit ( -1 );
        }
		
		
        ml = cd.getLocator ();


        if ( ml == null ) {
            System.err.println ( "Cannot build media locator from: " );
            System.exit ( 0 );
        }
        setBounds(0, 0, 350,350);
        open(ml);
		

    }

    public void setCodec( Codec pcodec ) {

        codec = pcodec;
    }

    /**
     * Given a media locator, create a processor and use that processor
     * as a player to playback the media.
     *
     * During the processor's Configured state, the RotationEffect is
     * inserted into the video track.
     *
     * Much of the code is just standard code to present media in JMF.
     *
     * @param  ml  Description of the Parameter
     * @return     Description of the Return Value
     */
    public boolean open ( MediaLocator ml ) {

System.out.println("ml = "+ml);

        try {
            p = Manager.createProcessor ( ml );
        } catch ( Exception e ) {
            System.err.println ( "Failed to create a processor from the given url: " + e );
            return false;
        }

        p.addControllerListener ( this );

        // Put the Processor into configured state.
        p.configure ();
        if ( !waitForState ( p.Configured ) ) {
            System.err.println ( "Failed to configure the processor." );
            return false;
        }

        // So I can use it as a player.
        p.setContentDescriptor ( null );

        // Obtain the track controls.
        TrackControl  tc[]          = p.getTrackControls ();

        if ( tc == null ) {
            System.err.println ( "Failed to obtain track controls from the processor." );
            return false;
        }

        // Search for the track control for the video track.
        TrackControl  videoTrack  = null;

System.out.println("tc leng = "+tc.length);

        for ( int i = 0; i < tc.length; i++ ) {
   		    System.out.println("tc["+i+"] format = "+tc[i].getFormat());
		    Format format = tc[i].getFormat();
            if ( format instanceof VideoFormat &&
			     format.toString().indexOf(defaultWidth) >= 0 && 
			     format.toString().indexOf(defaultHeight) >= 0  
			     ) {
                videoTrack = tc[i];
                break;
            }
        }

        if ( videoTrack == null ) {
            System.err.println ( "The input media does not contain a video track." );
            return false;
        }

        System.err.println ( "Video format: " + videoTrack.getFormat () );

        // Instantiate and set the frame access codec to the data flow path.
        try {
            Codec  codecs[]  = {codec};
            //			{new ImageJEffect ()};
            //			{new LibrasEffect ()};
            //,;

            videoTrack.setCodecChain ( codecs );
        } catch ( UnsupportedPlugInException e ) {
            System.err.println ( "The processor does not support effects." );
        }

        // Realize the processor.
        p.prefetch ();
        if ( !waitForState ( p.Prefetched ) ) {
            System.err.println ( "Failed to realize the processor." );
            return false;
        }





        //setBounds(0, 0,  486,384 );
        // Display the visual & control component if there's one.
        //getContentPane ().setLayout ( new BorderLayout () );

        Component     cc;
        Component     vc;
        if ( ( vc = p.getVisualComponent () ) != null ) {
            getContentPane ().add ( "Center", vc );
        }

        if ( ( cc = p.getControlPanelComponent () ) != null ) {
            getContentPane ().add ( "South", cc );
        }

        // Start the processor.
        p.start ();
        setVisible ( true );
        addWindowListener (
            new WindowAdapter () {
                public void windowClosing ( WindowEvent we ) {
                    p.close ();
                    System.exit ( 0 );
                }
            }
        );
        return true;
    }


    /**
     *  Adds a feature to the Notify attribute of the TestEffect object
     */
    public void addNotify () {
        super.addNotify ();
        pack ();
    }


    /**
     * Block until the processor has transitioned to the given state.
     * Return false if the transition failed.
     *
     * @param  state  Description of the Parameter
     * @return        Description of the Return Value
     */
    boolean waitForState ( int state ) {
        synchronized ( waitSync ) {
            try {
                while ( p.getState () != state && stateTransitionOK ) {
                    waitSync.wait ();
                }
            } catch ( Exception e ) {}
        }
        return stateTransitionOK;
    }


    /**
     * Controller Listener.
     *
     * @param  evt  Description of the Parameter
     */
    public void controllerUpdate ( ControllerEvent evt ) {

        if ( evt instanceof ConfigureCompleteEvent ||
                evt instanceof RealizeCompleteEvent ||
                evt instanceof PrefetchCompleteEvent ) {
            synchronized ( waitSync ) {
                stateTransitionOK = true;
                waitSync.notifyAll ();
            }
        } else if ( evt instanceof ResourceUnavailableEvent ) {
            synchronized ( waitSync ) {
                stateTransitionOK = false;
                waitSync.notifyAll ();
            }
        } else if ( evt instanceof EndOfMediaEvent ) {
            p.close ();
            System.exit ( 0 );
        }
    }



    /**
     * Main program
     *
	 * O cdigo abaixo NO  utilizado na plataforma SIGUS. Ele serviu
	 * apenas para testar o CameraReader fora da plataforma.
	 *
     * @param  args  The command line arguments
     */
    public static void main ( String[] args ) {

        MediaLocator       ml;
        CaptureDeviceInfo  cd;
        Vector             li;
        CaptureDeviceInfo   infoCaptureDevice;
        Format              arrFormats [];

        li = CaptureDeviceManager.getDeviceList ( new VideoFormat ( "YUV" ) );//RGB") );
        cd = (CaptureDeviceInfo) li.remove ( 0 );
        if ( cd == null ) {
            System.err.println ( "Cannot build media locator for camera" );
            System.exit ( -1 );
        }
        ml = cd.getLocator ();


        if ( ml == null ) {
            System.err.println ( "Cannot build media locator from: " );
            System.exit ( 0 );
        }

        CameraReader fa  = new CameraReader(null);

        if ( !fa.open ( ml ) ) {
            System.exit ( 0 );
        }
    }

}

