/** SigusApp.java:
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
 @author Hemerson Pistori (pistori@ec.ucdb.br)
 @created 18 de Maro de 2004
 
*/

package sigus.core;

import sigus.machineLearning.*;

import javax.swing.*;
import ij.process.*;

/**
  All applications that reads the input from a webcam and use SIGUS-weka
  integration module for learning should implement the SigusApp interface
*/
public interface SigusApp {


    public void init();
    /** Set window title */
    public void setGeneralTitle(String title);
    /** Indicate, with the word "playing" in the window title, that the application is NOT in learning mode (webcam interaction has begun) */
    public void setTitlePlaying();
    /** Process a visual recognized input, like a hand sign or an eye gaze.

        @param n - an integer associated to each different vision sign
    */
    public void process(int n);
    /** Set the class that will collect training samples and save them to
        an arff (weka) file.
    */
    public void setUpdater( Updater u);
    /** Set the ImageProcessor where Digital Image Processing will occurr (that
        window showing the segmented image) 
    */
    public void setImageProcessor( ImageProcessor i);
    /** True if application is in calibration mode (e.g.: collecting samples
        from face or hand skin), false otherwise.
    */
    public boolean getCalibrating();

}
