/** Learner_.java:
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
 @author Hemerson Pistori (pistori@ec.ucdb.br)
 @created 18 de Maro de 2004
 
*/

package sigus.machineLearning;

import sigus.parameterExtraction.*;


import ij.plugin.filter.PlugInFilter;
import java.awt.*;
import java.awt.image.*;
import java.util.Vector;
import java.io.*;
import ij.gui.*;
import ij.*;
import ij.process.*;
import ij.io.*;
import weka.core.*;
import weka.classifiers.*;
import weka.filters.*;
//import adaptree.*;


public class Learner_ implements PlugInFilter {

    ImagePlus imp;
    String trainingSetFileName = "trainingSet.arff";
    String modelFileName = "adaptree.mod";
    //private weka.classifiers.AdapTree classifier;
    private Classifier classifier;
    private Instances instances;
    private boolean isready = false;

    public int setup(String arg, ImagePlus imp) {
        this.imp = imp;
        return DOES_8G;
    }

    public void run(ImageProcessor ip) {

        init(false);

    }

    public void init(String ptraining) {
        trainingSetFileName = ptraining;
        init(false);
    }


    public boolean init(boolean readFileName) {

        if(readFileName)
            getFileName();

        IJ.showStatus("Opening Arff File...");
        if(!openArff(trainingSetFileName)) {
            IJ.showStatus("ERROR: Could not open "+trainingSetFileName);
            return false;
        }

        IJ.showStatus("Learning From Pre-Recorded Training Set...");
        learnFromInstances();

        if(classifier != null)
            IJ.showStatus("Learning Finished");
        else {
            IJ.write("Problem: Classifier was not built properly");
            return false;
        }

        IJ.register(Learner_.class);

        isready = true;
        return true;

    }


    public boolean isReady() {
        return(isready);
    }

    // Returns the region that is being focused
    public int getFocus(ImagePlus imp, ParameterExtractor_ pe) {

        int focus = 0;
        double pars[];

        if(instances != null) {

            pe.extract(imp);

            if(!pe.isBadSample()) {

                pars = pe.getParameters();


                // Tratar conversao real to string
                Instance inst = new Instance(ParameterExtractor_.MAX_PARS+1);
                inst.setDataset(instances);
                int i;
                for( i = 0 ; i < ParameterExtractor_.MAX_PARS ; ++i ) {
                    inst.setValue(i, (double) pars[i]);
                }
                inst.setValue(i,"1");

                try {
                    focus = (int) classifier.classifyInstance(inst);
                } catch (Exception e) {
                    System.err.println(e.getMessage());
                }


                return(focus+1);
            } else {
                return(0);
            }
        } else {
            return(0);
        }
    }



    public void learnInstance(ImagePlus imp, ParameterExtractor_ pe,String classValue ) {

        double pars[];

        if(pe==null) {
            System.out.println("ERROR - ParameterExtractor was not set");
            return;
        }


        pars = pe.getParameters();


        // Tratar conversao real to string
        Instance inst = new Instance(ParameterExtractor_.MAX_PARS+1);
        inst.setDataset(instances);
        int i;
        for( i = 0 ; i < ParameterExtractor_.MAX_PARS ; ++i ) {
            inst.setValue(i, (double) pars[i]);
        }
        inst.setValue(i,classValue);


        try {
            //          ((weka.classifiers.AdapTree)classifier).learnInstance(instances,inst);
            ((weka.classifiers.lazy.IBk) classifier).updateClassifier(inst);
        } catch (Exception e) {
            System.err.println(e.getMessage());
        }

    }


    void getFileName() {

        GenericDialog gd = new GenericDialog("Learn From Arff", IJ.getInstance());
        gd.addStringField("Arff File Name:", trainingSetFileName, 50);

        gd.showDialog();

        if (gd.wasCanceled()) {

            return;

        }
        trainingSetFileName = gd.getNextString();

    }

    void keepPluginOpened() {
        GenericDialog gd = new GenericDialog("Click OK to close this plugin", IJ.getInstance());
        gd.showDialog();
    }


    private boolean openArff(String fileName) {
        try {
            FileReader reader;
            // Loading Instances "Format" from an arff file.
            reader = new FileReader(fileName);
            instances = new Instances(reader);
            instances.setClassIndex(instances.numAttributes()-1);
        } catch (Exception e) {
            System.err.println(e.getMessage());
            return(false);
        }
        return(true);
    }


    public void learnFromInstances() {
        try {
            // Uncomment to Try AdapTree
            //		  classifier = new adaptree.AdapTree();
            //          ((weka.classifiers.AdapTree) classifier).setApplyFilter(true);
            //          ((weka.classifiers.AdapTree) classifier).setApplyInfoGain(true);
            //          ((weka.classifiers.AdapTree) classifier).setApplyAntiBayes(false);

            // Uncomment to Try Backpropagation
            //		  classifier = new weka.classifiers.neural.NeuralNetwork();

            // Uncomment to Try J48 (C4.5)
            //		  classifier = new weka.classifiers.trees.J48();
            // Uncomment to Try KNN
            classifier = new weka.classifiers.lazy.IBk(5);


            classifier.buildClassifier(instances);
        } catch (Exception e) {
            System.err.println(e.getMessage());
        }

    }


    private void loadClassifier() {
        try {
            // Loading Classifier
            InputStream is;
            is = new FileInputStream(modelFileName);
            ObjectInputStream objectInputStream = new ObjectInputStream(is);
            classifier = (Classifier) objectInputStream.readObject();
            objectInputStream.close();
        } catch (Exception e) {
            System.err.println(e.getMessage());
        }
    }



    void showAbout() {
        IJ.showMessage("Update Training Set",
                       "This plugin writes to a arff file some image parameters\n"+
                       "and a class value that will be used to construct a\n"+
                       "training example. This arff file can be further used\n"+
                       "by a learning algorithm. If the arff file already exist\n"+
                       "it will be just update, otherwise it will be created.\n"
                      );
    }


}
