/** Updater.java:
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
 @author Hemerson Pistori (pistori@ec.ucdb.br)
 @created 18 de Maro de 2004
 
*/

package sigus.machineLearning;

import sigus.parameterExtraction.*;


import ij.plugin.filter.PlugInFilter;
import java.awt.*;
import java.awt.image.*;
import java.util.Vector;
import java.io.*;
import ij.gui.*;
import ij.*;
import ij.process.*;
import ij.io.*;


/** Extracts features values from current Image and writes the
    result to a arff file */
abstract public class Updater implements PlugInFilter {

    ImagePlus imp;
    ImageProcessor ip;
    ParameterExtractor_ pe;
    private Learner_ learner = null;
    public String fileName = "trainingSet.arff";
    public String classValue;
    public String arffRelationName;
    public String arffClasses;
    static boolean needToCreateFile = true;
    static PrintWriter pw = null;
    static int totalInstances = 0;
    static boolean stopRequested = false;
    static boolean updating = false;

    public int setup(String arg, ImagePlus imp) {
        this.imp = imp;
        return DOES_8G;
    }

    abstract public void init();

    public void run(ImageProcessor pip) {
        int totalInstances = 0;
        ip = pip;
        readFileNameAndClassValue();
        update(classValue);
    }

    public void setImageProcessor(ImageProcessor pip) {
        ip = pip;
    }

    public void setImagePlus(ImagePlus pimp) {
        imp = pimp;
    }

    public void setParameterExtractor(ParameterExtractor_ pep) {
        pe = pep;
    }

    public void setLearner( Learner_ l) {
        learner = l;
    }

    public boolean isLearning() {
        if( learner != null)
            return true;
        else
            return false;
    }


    public void startUpdating(String pClassValue) {
        updating = true;
        classValue = pClassValue;
    }

    public void stopUpdating() {
        updating = false;
    }

    public boolean isUpdating() {
        return(updating);
    }

    public void setStopRequested(boolean value) {
        stopRequested = value;
    }

    public boolean getStopRequested() {
        return stopRequested;
    }

    public void update() {
        update(classValue);
    }


    public void update(String pClassValue) {

        //		byte[] pixels = (byte[])ip.getPixels();
        //		int width = ip.getWidth();
        //		Rectangle r = ip.getRoi();
        //      int offset, i, attCounter;

        if(pe==null) {
            System.out.println("ERROR - ParameterExtractor was not set");
            return;
        }

        int i;
        FileOutputStream fos = null;

        IJ.showStatus("Updating Arff file...");
        String ls = System.getProperty("line.separator");


        if(needToCreateFile) {
            try {
                FileInputStream fis = new FileInputStream(fileName);
                if(fis != null) { // File already exist
                    needToCreateFile = false;
                    fis.close();
                }
            } catch (IOException e) {}
            try {
                pw = null;
                fos = new FileOutputStream(fileName,true);
                BufferedOutputStream bos = new BufferedOutputStream(fos);
                pw = new PrintWriter(bos);
                if(needToCreateFile) {
                    // Save Arff Reader.
                    pw.print("@relation "+arffRelationName+ls+ls);
                    for( i = 1; i <= ParameterExtractor_.MAX_PARS; ++i ) {
                        pw.print("@attribute "+pe.getName(i)+" real"+ls);
                    }
                    pw.print("@attribute class {"+arffClasses+"}"+ls);
                    pw.print("@data"+ls+ls);
                    needToCreateFile = false;
                }
            } catch (IOException e) {
                IJ.write("" + e);
                return;
            }
        }

        if(!pe.isBadSample()) {

            double[] pars = pe.getParameters();

            for(i = 0 ; i < ParameterExtractor_.MAX_PARS ; ++i ) {
                pw.print(pars[i] + " ");
            }

            ++totalInstances;


            //           IJ.beep();

            pw.print(classValue+ls);
            pw.flush();

            if(learner!=null)
                learner.learnInstance(imp,pe,classValue);


            System.out.println("  "+totalInstances+"Examples Collected (current class = "+classValue+")");
        } else {
            //		   IJ.showStatus("Bad Sample");
            //		   System.out.println("bad ...");
        }
        IJ.register(Updater.class);
    }

    public void clearTrainingSet() {
        (new File(fileName)).delete();
        totalInstances = 0;
        needToCreateFile = true;

    }

    public int getTotalInstances() {
        return(totalInstances);
    }

    public String getClassValue() {
        return classValue;
    }

    public String getFileName() {
        return fileName;
    }

    boolean readFileNameAndClassValue() {

        GenericDialog gd = new GenericDialog("Update Arff FileROI Size", IJ.getInstance());
        gd.addStringField("File Name:", fileName, 50);
        gd.addStringField("Class Value:", classValue, 3);

        gd.showDialog();

        if (gd.wasCanceled()) {
            return(false);

        }
        fileName = gd.getNextString();
        classValue = gd.getNextString();
        return(true);
    }



    void showAbout() {
        IJ.showMessage("Update Training Set",
                       "This plugin writes to a arff file some image parameters\n"+
                       "and a class value that will be used to construct a\n"+
                       "training example. This arff file can be further used\n"+
                       "by a learning algorithm. If the arff file already exist\n"+
                       "it will be just update, otherwise it will be created.\n"
                      );
    }


}
