/** ImageMomentsTracking_.java:
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
 @author Hemerson Pistori (pistori@ec.ucdb.br)
 @created 18 de Maro de 2004
 
*/

package sigus.tracking;

import sigus.*;

import ij.*;
import ij.plugin.filter.PlugInFilter;
import ij.process.*;
import java.awt.*;
import ij.gui.*;

/**
  ImageJ plugin filter that creates a ROI containing the target object (hand, face) using Image Moments. Assumes that the image has been previously segmented, and only the target pixels are not black. 
*/
public class ImageMomentsTracking_ implements PlugInFilter {

    private int ignoreMargin = 5;
    private static int erros = 0;
    private double stdMult = 3;

    ImagePlus imp;

    public int setup(String arg, ImagePlus imp) {
        this.imp = imp;
        if (arg.equals("about")) {
            showAbout();
            return DONE;
        }
        return DOES_8G+DOES_STACKS+SUPPORTS_MASKING;
    }

    public void run(ImageProcessor ip) {


        ip.resetRoi();
        Rectangle r = ip.getRoi();


        double centerx = 0;
        double centery = 0;
        double stdvx = 0;
        double stdvy = 0;
        int totpixels = 0;

        //System.out.println("r.x = "+r.x+" r.y = "+r.y+" r.width = "+r.width+" r.height = "+r.height);
        // Calculate means
        for (int x=r.x+ignoreMargin;x<(r.width-ignoreMargin);++x) {
            for (int y=r.y+ignoreMargin;y<(r.height-ignoreMargin);++y) {
                if(ip.getPixel(x,y) == 0) {
                    //				System.out.print(" "+x+" "+y+" " );
                    //					System.out.print("PEGOU");
                    centerx = centerx + x;
                    centery = centery + y;
                    ++totpixels;
                }
            }
        }


        if(totpixels != 0) {

            centerx = centerx/totpixels;
            centery = centery/totpixels;


            // Calculate Standard Deviations
            for (int x=r.x+ignoreMargin;x<(r.width-ignoreMargin);++x) {
                for (int y=r.y+ignoreMargin;y<(r.height-ignoreMargin);++y) {
                    if(ip.getPixel(x,y) == 0) {
                        stdvx = stdvx + Math.abs(x - centerx);
                        stdvy = stdvy + Math.abs(y - centery);
                    }
                }
            }


            stdvx = (stdvx / (totpixels-1))*stdMult;
            stdvy = (stdvy / (totpixels-1))*stdMult;
            //	      System.out.println("cx = "+centerx+" cy = "+centery+ " sigx = "+stdvx+" sigy = "+stdvy+" totpix = "+totpixels);
            imp.setRoi((int)(centerx-stdvx),(int)(centery-stdvy),(int)(2*stdvx),(int)(2*stdvy));
            //imp.setRoi((int)(centerx-5),(int)(centery-5.0),(int)(centerx+5.0),(int)(centery+5.0));
            //imp.setRoi(5,5,20,50);
            showCenter(ip,centerx,centery);
        } else {
            System.out.print("{"+(++erros)+"}");
        }



    }


    void showCenter(ImageProcessor ip, double centerx, double centery) {

        for(int i = (int)centerx-10 ; i < (int)centerx + 10; ++i ) {
            for(int j = (int)centery-10 ; j < (int)centery + 10; ++j ) {
                ip.putPixelValue(i,j,50.0);
            }
        }
    }

    void showAbout() {
        IJ.showMessage("About HandsRegion_...",
                       "Detect Hands Regions (In the future)."
                      );
    }
}

